/*********************************************************************/
/*                   Solomon Systech Limited                         */
/*********************************************************************/
/* Program Name :io.c
/* Description : Define io routines for programming Zilog Z8 Encore! eZ8F642c
/*               8-bit MCU to drive SSD1926 DVK board
/*				 This souce code provides a reference design for customers who intend
/*				 to use any 8-bit MCU to control SSD1926 DVK board
/*				 The IO routine refer to read and write to register using byte/word/long method
/*				 
/* Author : Aik Hong (SSL Singapore FAE)
/*********************************************************************/

/*********************************************************************/
/*                        Change Log                                 */
/*********************************************************************/
/* 13 March 2008: Initial Revision
/* Modifications :
/* 1.0 : Version 1 
/* 1.1 :
/*********************************************************************/


#include "Header_file.h"
#include "io.h"
//#include "ssd192X.h"

//#include "memory.h"
//#include "lcd_hw.h"
//#include "lcd.h"

//extern LCD_Offset;


char SSD_REGRB(UINT32 regNum)
{
	UINT8 temp_reg, temp_reg1, receive;
	UINT32 value1;
	int count;

	value1 = regNum;
	count = 0;

	value1 = (value1 & 0x0000FF00);
	temp_reg = value1 >>8;

	value1 = regNum;
	temp_reg1 = value1 & 0x000000FF;
	
	write_location(0x00);		// bit 7 = 0, means register access
	write_location(temp_reg);
	write_location(temp_reg1);
	
	// set PB as input 
 	PBADDR 	= 0x01;     /* PB Data Dir */
	PBCTL 	= 0xFF;  

	receive = 0x00;

	SET_CS_BIT;	
	SET_DC_BIT; 		/* It's a data */
	CLEAR_E_BIT; 		
	SET_RW_BIT; 

	while (count <2 )
	{
		CLEAR_CS_BIT;
		delay_us(3); 		/* important delay needed */
		receive = PBIN;
		SET_CS_BIT;
		delay_us(1); 				
		count ++;
	}
	
	SET_CS_BIT;
	

	PBADDR 	= 0x01;     // PB Data Dir 
	PBCTL 	= 0x00;    	// PB Data Dir = output 
  	PBADDR 	= 0x03;     // PB Output Ctrl 
  	PBCTL 	= 0x00;    	// PB Out Ctrl = push-pull 

  	PBOUT	= 0x00;     // Init to 0 first 

	// Read burst terminal
	/*
	write_data(0x00);
	write_data(0x00);
	write_data(0x00);
	*/
	
	return(receive);

}

UINT16 SSD_REGRW(UINT32 regNum)
{
	UINT8 get[3];
	UINT16 receive;
	int count =0;

	while ( count < 2)
	{
		get[count] = SSD_REGRB(regNum+count);
		count++;
	}

	receive = get[1];
    receive = receive << 8;
	receive = receive | get[0] ;
		
	return(receive);
}

UINT32 SSD_REGRL(UINT32 regNum)
{
	UINT8 get[5];
	UINT32 long_value=0, receive=0;
	
	int count=0;

	while ( count < 4)
	{
		get[count] = SSD_REGRB(regNum+count);
		count++;
	}

	receive = get[3];
    receive = receive << 24;
	long_value = receive;
	receive = get[2];
	receive = receive << 16;
	long_value  = long_value | receive ;
	receive = get[1];
	long_value = long_value | receive << 8;
	receive = get[0];
	long_value = long_value | receive;
		
	return(long_value);
}

char SSD_MEMRB(UINT32 memAddr)
{
	UINT8 temp_add, temp_add1, receive, burst_end;
	UINT32 value1;
	int count = 0;

	value1 = memAddr;
	value1 = (value1 & 0x00030000);
	value1 = value1 >>16;
	temp_add =  value1 | 0x80; 
	write_location(temp_add);		// bit 7 = 1, means memory access

	value1 = memAddr;
	value1 = (value1 & 0x0000FF00);
	temp_add = value1 >>8;
	write_location(temp_add);

	value1 = memAddr;
	temp_add = value1 & 0x000000FF;
	write_location(temp_add);
	
	// set PB as input 
 	PBADDR 	= 0x01;     /* PB Data Dir */
	PBCTL 	= 0xFF;  

	receive = 0x00;

	SET_CS_BIT;	
	SET_DC_BIT; 		/* It's a data */
	CLEAR_E_BIT; 		/* signal on the data port PB is latched now, E bit = RD bit */
	SET_RW_BIT; 
	CLEAR_CS_BIT;	

	while (count <2 )
	{
		delay_us(5); 		/* important delay needed */
		receive = PBIN;
		SET_CS_BIT;
		CLEAR_CS_BIT;	
		count ++;
	}
	
	SET_CS_BIT;

	PBADDR 	= 0x01;     /* PB Data Dir */
	PBCTL 	= 0x00;    	/* PB Data Dir = output */
  	PBADDR 	= 0x03;     /* PB Output Ctrl */
  	PBCTL 	= 0x00;    	/* PB Out Ctrl = push-pull */

  	PBOUT	= 0x00;     /* Init to 0 first */

	// Read burst terminal
	/*
	write_data(0x00);
	write_data(0x00);
	write_data(0x00);
	*/

	return(receive);

}

UINT16 SSD_MEMRW(UINT32 memAddr)
{
	UINT8 get[3];
	UINT16 receive;
	int count =0;

	while ( count < 2)
	{
		get[count] = SSD_MEMRB(memAddr+count);
		count++;
	}

	receive = get[1];
    receive = receive << 8;
	receive = receive | get[0] ;
		
	return(receive);
}


void write_location(UINT8 Reg_Mem)
{
	// location refer to mem address or the reg value
	PBOUT = Reg_Mem;

	CLEAR_DC_BIT; 	/* set reg */
	CLEAR_RW_BIT;
	SET_E_BIT;
	CLEAR_CS_BIT;
	delay_us(1); 	// must have delay to ensure value is put in	
	SET_CS_BIT; 
}

void write_data(UINT8 value)
{
	PBOUT = value;

	SET_DC_BIT; 	/* set value */
	CLEAR_RW_BIT;
	SET_E_BIT;
	CLEAR_CS_BIT;
	delay_us(1); // must have delay to ensure value is put in	
	SET_CS_BIT; 

}

void SSD_REGWB(UINT32 regNum,UINT8 value)
{

	UINT32 reg_2;
	UINT32 Store_regNum;

	// write the 1st set of 8 bits
	Store_regNum = regNum;
	//reg_2= Store_regNum >> 24;
	reg_2 = 0x00;	// 1 register write, Bit 7 = 0, since max register 0x0000FFFF
 	write_location(reg_2);	

	// write the 3rd set of 8 bits
	Store_regNum = regNum;
	Store_regNum = Store_regNum & 0x0000FF00;
	reg_2= Store_regNum >> 8;
 	write_location(reg_2);	

	// write the 4th set of 8 bits
	Store_regNum = regNum;
	reg_2 = Store_regNum & 0x000000FF;
 	write_location(reg_2);	

	write_data(value);
}

void SSD_REGWW(UINT32 regNum,UINT16 value)
{
	UINT32 reg_2;
	UINT32 Store_regNum;
	UINT16 Store_value; 
	UINT8 data;

	// example regnum = 0x71, value = 0x0123
	// 0x71 = 0x23
	// 0x72 = 0x01

	// write the 1st set of 8 bits
	//Store_regNum = regNum;
	reg_2 = 0x00;	// 1 register write, Bit 7 = 0, since max register 0x0000FFFF
 	write_location(reg_2);	

	// write the MSB of the register address
	Store_regNum = regNum;
	Store_regNum = Store_regNum & 0x0000FF00;
	reg_2= Store_regNum >> 8;
 	write_location(reg_2);	

	// write the LSB of the register address
	Store_regNum = regNum;
	reg_2 = Store_regNum & 0x000000FF;
 	write_location(reg_2);	

	// write the 8 bits value
	Store_value = value;
	data = Store_value & 0x00FF;

	write_data(data);

	// write the next 8 bit value into the next register
	Store_value = value;
	Store_value = Store_value & 0xFF00;
	data = Store_value >> 8;
	
	write_data(data);
}

void SSD_REGWL(UINT32 regNum,UINT32 value)
{
	UINT32 reg_2;
	UINT32 Store_regNum;
	UINT32 Store_value; 
	UINT8 data;

	// example regnum = 0x71, value = 0x01234567
	// 0x71 = 0x67
	// 0x72 = 0x45
	// 0x73 = 0x23
	// 0x74 = 0x01

	// write the 1st set of 8 bits
	//Store_regNum = regNum;
	reg_2 = 0x00;	// 1 register write, Bit 7 = 0, since max register 0x0000FFFF
 	write_location(reg_2);	

	// write the MSB of the register address
	Store_regNum = regNum;
	Store_regNum = Store_regNum & 0x0000FF00;
	reg_2= Store_regNum >> 8;
 	write_location(reg_2);	

	// write the LSB of the register address
	Store_regNum = regNum;
	reg_2 = Store_regNum & 0x000000FF;
 	write_location(reg_2);	

	// write the 8 bits value
	Store_value = value;
	data = Store_value & 0x000000FF;

	write_data(data);

	// write the next 8 bit value into the next register
	Store_value = value;
	Store_value = Store_value & 0x0000FF00;
	data = Store_value >> 8;

	write_data(data);

	// write the next 8 bit value into the next register
	Store_value = value;
	Store_value = Store_value & 0x00FF0000;
	data = Store_value >> 16;

	write_data(data);

	// write the next 8 bit value into the next register
	Store_value = value;
	Store_value = Store_value & 0xFF000000;
	data = Store_value >> 24;
	
	write_data(data);
}


void SSD_MEMWB(UINT32 memAddr,UINT8 value)
{
	//printf("REGWB[%x]=%x\n",regNum,value);
	UINT8 mem;
	UINT32 Store_memAddr;
	UINT32 i;

	// write the 1st set of 8 bits
	Store_memAddr = memAddr;
	mem = Store_memAddr >> 16;
	mem = mem | 0x80;	// since mem access, first bit must be 1
 	write_location(mem);	

	// write the 3rd set of 8 bits
	Store_memAddr = memAddr;
	Store_memAddr = Store_memAddr & 0x0000FF00;
	mem= Store_memAddr >> 8;
 	write_location(mem);	

	// write the 4th set of 8 bits
	Store_memAddr = memAddr;
	mem = Store_memAddr & 0x000000FF;
 	write_location(mem);

	write_data(value);
}

void SSD_MEMWW(UINT32 memAddr,UINT16 value)
{
	UINT8 mem;
	UINT32 Store_memAddr;
	UINT16 Store_value; 
	UINT8 data;

	// example regnum = 0x71, value = 0x0123
	// 0x71 = 0x23
	// 0x72 = 0x01

	// write the 1st set of 8 bits
	Store_memAddr = memAddr;
	mem = Store_memAddr >> 16;
	mem = 0x80|mem;	// memory write, Bit 7 = 1
 	write_location(mem);	

	// write the MSB of the memory address
	Store_memAddr = memAddr;
	Store_memAddr = Store_memAddr & 0x0000FF00;
	mem= Store_memAddr >> 8;
 	write_location(mem);	

	// write the LSB of the register address
	Store_memAddr = memAddr;
	mem = Store_memAddr & 0x000000FF;
 	write_location(mem);	

	// write the 8 bits value
	Store_value = value;
	data = Store_value & 0x00FF;

	write_data(data);

	// write the next 8 bit value into the next register
	Store_value = value;
	Store_value = Store_value & 0xFF00;
	data = Store_value >> 8;
	
	write_data(data);
}


