/*********************************************************************/
/*                   Solomon Systech Limited                         */
/*********************************************************************/
/* Program Name :sdcmd.c
/* Description : Define SD routines for programming Zilog Z8 Encore! eZ8F642c
/*               8-bit MCU to drive SSD1926 DVK board
/*				 This souce code provides a reference design for customers who intend
/*				 to use any 8-bit MCU to control SSD1926 DVK board
/*				 The routine here define on sending commands to SD card, like SD_read/write/select
/*				 
/* Author : Aik Hong (SSL Singapore FAE)
/*********************************************************************/

/*********************************************************************/
/*                        Change Log                                 */
/*********************************************************************/
/* 13 March 2008: Initial Revision
/* Modifications :
/* 1.0 : Version 1 
/* 1.1 :
/*********************************************************************/

#include "sdcmd.h"
//#include "ssd192X.h"
#include "io.h"

extern UINT16 SD_RCA;

UINT32 sector_count;
UINT32 erase_size;

///////////////////////////////////////////////////////////////////////////////////
// SD card subroutines
///////////////////////////////////////////////////////////////////////////////////

UINT32 get_cardstatus(void) {

    UINT32 status;

	status = SSD_REGRW(0x1112)<<16 + SSD_REGRW(0x1110);
	status = SSD_REGRL(0x1110);

    SSD_REGWB(0x112F, 0x02);       //soft cmd reset
    while(SSD_REGRB(0x112F)&0x02); //wait till soft cmd rst complt

    //Normal interrupt handling
    SSD_REGWB(0x1134,SSD_REGRB(0x1134)|0x01); //enable read interrupt
    //SSD_REGWB(0x1138,SSD_REGRB(0x1138)|0x23); //enable read interrupt
    SSD_REGWB(0x1130,0x01); //clear previous interrupt

    //Error interrupt handling
    SSD_REGWW(0x1136, SSD_REGRB(0x1136)|0x018F); //enable error interrupt
    //SSD_REGWW(0x113A, SSD_REGRB(0x113A)|0x0000); //enable error interrupt
    SSD_REGWW(0x1132, 0x718F);

    while(!SD_Send_CMD(CMD13, ((UINT32)SD_RCA)<<16, r1_resp|cmd_check));
    while(!SSD_REGRB(0x1130)&0x01);
    while(SSD_REGRB(0x1132)|0xFE == 0xFF);
    
    while(!SSD_REGRB(0x1130)&0x01); //check cmd complt int
    SSD_REGWB(0x1130, 0x01); //clear previous int
    while(SSD_REGRW(0x1132)); //check error int
    SSD_REGWW(0x1132, 0x718F); //clear error int


    //status = SSD_REGRW(0x1112)<<16 + SSD_REGRW(0x1110);
	status = SSD_REGRL(0x1110);
    //display_resp(6);

    return status;
}

UINT8 Select_card(void) {

    SSD_REGWB(0x1130, 0x01); 		// clear previous interrupt
    SSD_REGWB(0x1132, 0x23); 		// clear previous interrupt
    //select card command -> change to transfer state
    while(!SD_Send_CMD(CMD7, ((UINT32)SD_RCA)<<16, r1b_resp|cmd_check));
    while(!SSD_REGRB(0x1130)&0x01);
    while(SSD_REGRB(0x1132)|0xFE == 0xFF);
    SSD_REGWB(0x1130, 0xFF); // clear previous interrupt
    SSD_REGWB(0x1132, 0xFF); // clear previous interrupt

    return 0;
}

UINT8 Deselect_card(void) {

    SSD_REGWB(0x1130, 0x01); // clear previous interrupt
    SSD_REGWB(0x1132, 0x23); // clear previous interrupt
    //select card command -> change to transfer state
    while(!SD_Send_CMD(CMD7, 0x00, cmd_check));
    while(!SSD_REGRB(0x1130)&0x01);
    //display_resp(7);
    SSD_REGWB(0x1130, 0xFF); // clear previous interrupt
    SSD_REGWB(0x1132, 0xFF); // clear previous interrupt

	return 0;

}

//UINT8 SD_getDriveSize(hwInterface *iface, UINT32* drive_size) {
UINT8 SD_getDriveSize(UINT32* drive_size) {

    UINT8 cardresp, i, by;
    UINT8 iob[16];
    UINT16 c_size, c_size_mult, read_bl_len;

    SSD_REGWB(0x1134, SSD_REGRB(0x1134)|0x01); // enable read interrupt
    SSD_REGWB(0x1136, SSD_REGRB(0x1136)|0x01); // timeout error enable
    SSD_REGWB(0x1138, SSD_REGRB(0x1138)|0x01); // enable read interrupt
    SSD_REGWB(0x113A, SSD_REGRB(0x113A)|0x01); // timeout error interrupt
    SSD_REGWB(0x1130, 0x01); // clear previous interrupt
    SSD_REGWB(0x1132, 0x23); // clear previous interrupt

    //send card status
    while(!SD_Send_CMD(CMD13, ((UINT32)SD_RCA)<<16, r1_resp|cmd_check));
    while(SSD_REGRB(0x1132)|0xFE == 0xFF);
    while(!SSD_REGRB(0x1130)&0x01);
    //display_resp(3);

    SSD_REGWB(0x1130, 0x01); // clear previous interrupt
    SSD_REGWB(0x1132, 0x23); // clear previous interrupt
    while(!SD_Send_CMD(CMD9, ((UINT32)SD_RCA)<<16, r2_resp|cmd_crc_chk));
    //while(!SD_Send_CMD(CMD9, 0, r2_resp|cmd_check));
    while(SSD_REGRB(0x1132)|0xFE == 0xFF);
    while(!SSD_REGRB(0x1130)&0x01);
    //while(SSD_REGRB(0x1124)&0x01); // check if command sent completed
    ////display_resp(7);


    //get the C_SIZE inside CSD register
    c_size = iob[6]&0xC0;
    c_size >>= 6;
    c_size += ((UINT16)iob[7])<<2;
    c_size += (((UINT16)iob[8])&0x03)<<10;

    //get the READ_BL_LEN inside CSD register
    by = iob[9] & 0x0F;
    read_bl_len = 1;
    read_bl_len <<= by;

    //get the C_SIZE_MULT inside CSD register
    by = iob[5] & 0x03;
    by <<= 1;
    by += iob[4] >> 7;
    c_size_mult = 1;
    c_size_mult <<= (2+by);

    *drive_size = (UINT32)(c_size+1) * (UINT32)c_size_mult * (UINT32)read_bl_len;
	
    return 0;
}

UINT8 SD_csd(void) {
// CSD = Card Specific Data

    UINT8 cardresp, i, by;
    UINT8 iob[16];
    UINT16 c_size, c_size_mult, read_bl_len;
    UINT32 temp_size;

    SSD_REGWB(0x1134, SSD_REGRB(0x1134)|0x01); // enable read interrupt
    SSD_REGWB(0x1136, SSD_REGRB(0x1136)|0x01); // timeout error enable
    SSD_REGWB(0x1138, SSD_REGRB(0x1138)|0x01); // enable read interrupt
    SSD_REGWB(0x113A, SSD_REGRB(0x113A)|0x01); // timeout error interrupt
    SSD_REGWB(0x1130, 0x01); // clear previous interrupt
    SSD_REGWB(0x1132, 0x23); // clear previous interrupt

    //send card status
    while(!SD_Send_CMD(CMD13, ((UINT32)SD_RCA)<<16, r1_resp|cmd_check));	// request card to send status
    while(SSD_REGRB(0x1132)|0xFE == 0xFF);
    while(!SSD_REGRB(0x1130)&0x01);
    ////display_resp(3);

    SSD_REGWB(0x1130, 0x01); // clear previous interrupt
    SSD_REGWB(0x1132, 0x23); // clear previous interrupt
    while(!SD_Send_CMD(CMD9, ((UINT32)SD_RCA)<<16, r2_resp|cmd_crc_chk));	// CMD9 = Card Specific data in CSD register
    while(SSD_REGRB(0x1132)|0xFE == 0xFF);
    while(!SSD_REGRB(0x1130)&0x01);
    ////display_resp(7);

    //get the C_SIZE inside CSD register
    c_size = iob[6]&0xC0;
    c_size >>= 6;
    c_size += ((UINT16)iob[7])<<2;
    c_size += (((UINT16)iob[8])&0x03)<<10;

    //get the READ_BL_LEN inside CSD register
    by = iob[9] & 0x0F;
    read_bl_len = 1;
    read_bl_len <<= by;

    //get the C_SIZE_MULT inside CSD register
    by = iob[5] & 0x03;
    by <<= 1;
    by += iob[4] >> 7;
    c_size_mult = 1;
    c_size_mult <<= (2+by);

    temp_size = ((UINT32)(c_size+1) * (UINT32)c_size_mult * (UINT32)read_bl_len);
    sector_count = temp_size/512;
    if( (temp_size%512) != 0) {
        sector_count--;
    }

    erase_size = iob[4] & 0x3F;
    erase_size <<= 1;

    erase_size += iob[3] >> 7;
    
    return 0;
}

UINT8 SD_read(UINT32 LBA_address,UINT8* buf,UINT8 count) {
//UINT8 SD_read(UINT32 LBA_address,UINT8 count1) {

    UINT16 i, j;
	UINT8 k_cnt;
	
	j = 0;

    while(count) {
		
		k_cnt = count;

        SSD_REGWB(0x112F, 0x04);       // soft reset
        while(SSD_REGRB(0x112F)&0x04); // wait till soft rst complt
        while(SSD_REGRL(0x1124)&0x02); // check if DAT inhibit

		 
	    SSD_REGWB(0x110C, 0x10); // transfer mode register single read
		//SSD_REGWB(0x110C, 0x00); // transfer mode register
		//SSD_REGWB(0x110C, 0x10); // transfer mode register
		
		SSD_REGWB(0x1104, 0x00); // write block size
        SSD_REGWB(0x1105, 0x02); // write block size
		
       	//Normal interrupt handling
    	SSD_REGWB(0x1134,SSD_REGRB(0x1134)|0x23); // enable read interrupt
        //SSD_REGWB(0x1138,SSD_REGRB(0x1138)|0x23); // enable read interrupt	
        SSD_REGWB(0x1130,0x23); // clear previous interrupt

       	//Error interrupt handling
       	SSD_REGWW(0x1136, SSD_REGRB(0x1136)|0x0000); //enable error interrupt
       	//SSD_REGWW(0x113A, SSD_REGRB(0x113A)|0x0000); //enable error interrupt
       	SSD_REGWW(0x1132, 0x01FF);
       	
		while(!SD_Send_CMD(CMD17,LBA_address*512,r1_resp|data_present|cmd_check));
		
       	while(!SSD_REGRB(0x1130)&0x01); //chck cmd complte int
       	SSD_REGWB(0x1130, 0x01); //clear previous interrupt

       	while(!SSD_REGRB(0x1130)&0x20);
       	SSD_REGWB(0x1130,0x20);

	  	if(SSD_REGRB(0x1125)&0x08)
		{
			// start to read in from the read buffer 
			delay_us(10);
			i=0;
       		while(i<512) {
       		    buf[i] = SSD_REGRB(0x1120);
                	++i;
            		}
       }
       	SSD_REGWB(0x1130,0x20);
       	while(!SSD_REGRB(0x1130)&0x02); 	//check Transfer complt int
       	SSD_REGWB(0x1130, 0x02); 			// clear previous interrupt

       	while(SSD_REGRW(0x1132))
		{
			j++;
			if( j = 0x400)
			{
				break;
			}
		}

        --count;
        ++LBA_address;
    }
    return 0;
}

UINT8 SD_write(UINT32 LBA_address,UINT8* buf,UINT8 count) {
//UINT8 SD_write(UINT32 LBA_address,UINT8 count) {

    UINT16 i,j;

    while(count) {

        while(!get_cardstatus()&0x0100);

        SSD_REGWB(0x112F, 0x04);       //soft DAT reset
        while(SSD_REGRB(0x112F)&0x04); //wait till soft DAT rst complt
        while(SSD_REGRL(0x1124)&0x02); //check if DAT inhibit
            
        SSD_REGWB(0x1104, 0x00); //write block size
        SSD_REGWB(0x1105, 0x02); //write block size
        SSD_REGWB(0x110C, 0x00); //transfer mode register
    
        //Normal interrupt handling
        SSD_REGWB(0x1134, SSD_REGRB(0x1134)|0x13); //enable write interrupt
        //SSD_REGWB(0x1138, SSD_REGRB(0x1138)|0x13); //enable write interrupt
        SSD_REGWB(0x1130, 0x13); //clear all type of normal interrupt
    
        //Error interrupt handling
        //SSD_REGWW(0x1136, SSD_REGRB(0x1136)|0x0000); //enable error interrupt
        //SSD_REGWW(0x113A, SSD_REGRB(0x113A)|0x0000); //enable error interrupt
        SSD_REGWW(0x1132, 0x01FF); //clear all type of error interrupt
    
        while(!SD_Send_CMD(CMD24,LBA_address*512,r1_resp|data_present|cmd_check));
        while(!SSD_REGRB(0x1130)&0x01); 	//check if command complete
        SSD_REGWB(0x1130, 0x01); 			//clear previous interrupt
        while(!SSD_REGRB(0x1130)&0x10); 	//check Buffer write int
        SSD_REGWB(0x1130, 0x10); 			//clear previous interrupt
    
        i=j=0;
        while(i<512) {
            //check if Write Enable bit set, which mean buffer is free for new data
            if(SSD_REGRB(0x1125)&0x04) {
                //put data that need be written into SSD1928 buffers
                SSD_REGWB(0x1120, *buf++);
                ++i;
            }
            //death loop check interrupt timer
        }
    
        while(!SSD_REGRB(0x1130)&0x02); //check Transfer complt int
        SSD_REGWB(0x1130, 0x02); //clear previous interrupt
        --count;
        ++LBA_address;
    }

    return 0;
}


